#include <stdio.h>
#include <stdlib.h>
#include <math.h>
//------------------------------------------------------------------------------
// Система этанол-вода. Расчет спиртуозности (в мольных долях) от температуры 
// кипения и давления. Коэффициенты активности рассчитываются в приближении 
// Ван-Лаара (van Laar).
// Для любого компиллятора C стандарта C99 и более поздних. 
// Пример компилляции для gcc из командной строки:
// gcc -std=c99 -o van_Laar_xy van_Laar_xy.c -lm
//------------------------------------------------------------------------------
// OldBean, 22.03.2017
//------------------------------------------------------------------------------
// Давление насыщенного пара - по уравнению Антуан
float A1 = 8.21330, B1 = 1652.050, C1 = 231.480; // Этанол
float A2 = 8.07131, B2 = 1730.63, C2 = 233.426; // Вода
float Psat1(float T) {
  return powf(10.0, A1 - B1/(T + C1));
}
float Psat2(float T) {
  return powf(10.0, A2 - B2/(T + C2));
}
//------------------------------------------------------------------------------
// Коэффициенты активности по Ван Лаару (van Laar)
# define M_E 2.7182818284590452354
float A12 = 1.6798, A21 = 0.9227;
float Gam1(float xa) { // Этанол
  float wrk = A21*(1.0 - xa)/(A12*xa + A21*(1.0 - xa));
  return powf(M_E, A12*wrk*wrk);
}
float Gam2(float xa) { // Вода
  float wrk = A12*xa/(A21*(1.0 - xa) + A12*xa);
  return powf(M_E, A21*wrk*wrk);
}
//------------------------------------------------------------------------------
float P = 760.0; // Давление
float T = 85.0;  // Температура
//------------------------------------------------------------------------------
// Модифицированный закон Рауля (с активностями)
float Raoult(float xa) {
  return Psat1(T)*xa*Gam1(xa) + Psat2(T)*(1.0 - xa)*Gam2(xa) - P; 
}
//------------------------------------------------------------------------------
// Вычисление корня методом дихотомии
float my_root(float aa, float bb, float eps, float its) {
  float a, b;
  if(aa < bb) {a = aa; b = bb;}
  else {a = bb; b = aa;}
  float fa = Raoult(a), fb = Raoult(b);
  if(fa*fb > 0) {
    fprintf(stderr, "В заданном интервале нет корней (или больше одного корня)\n");
    return 0;
  }
  int it = 0; 
  float dx = b - a;
  while(it < its && dx > eps) {
    float c = 0.5*(a + b); 
    float fc = Raoult(c);
    if(fa*fc >= 0) {a = c; fa = fc;}
    else {b = c; fb = fc;}
    dx = b - a; it += 1;
  }
  return 0.5*(a + b);
}
//------------------------------------------------------------------------------
int main(int argc, char **argv) {
  if(argc > 1) P = atof(argv[1]);
  printf("P = %.1f, mmHg\n", P);
  for(int i = 0; i < 22; i++) {
    T = 100 - i*1.0;
    float xa = my_root(0.0, 1.0, 1.0E-6, 30);
    float ya = Psat1(T)*xa*Gam1(xa)/P;
    printf("%.3f\t%.3f\t%.3f\n", T, xa, ya);
  }
  return 0;
}
//------------------------------------------------------------------------------
